	name	FlipDRV
	title	Flip Antennas on Their Backs
;
;  This program chains in on top of a KCT-compatible TSR.
;
;  It effectively adds two interrupt call interfaces to the 
;  RotorDRV facility:
;     AH = 0x7F     Set Flip Antennas Status
;     AL = 0x00      flip mode off (normal operation)
;     AL = 0x01      flip mode on  (inverted antennas)
;
;     AH = 0x7E     Get Flip Antennas Status
;   returns:
;     AL = 0x00      flip mode is off (normal operation)
;     AL = 0x01      flip mode is on  (inverted antennas)
;
;  When flip mode is on and a call to the KCT to set or read
;  target angles is made, FlipDRV modifies the call accordingly
;  to get inverted antenna angles.
;

_TEXT	segment word public 'CODE'

	ORG	100h			; this is a .COM file!

	assume	cs:_TEXT, ds:_TEXT, es:_TEXT, ss:_TEXT

start:	jmp	install

;******************************************************************
;  Here is the interrupt service routine.
;******************************************************************

	assume	ds:nothing, es:nothing, ss:nothing

isr:	jmp	dispatch

	ORG	isr + 4

old_isr		DD	?	; old RotorDRV address

	ORG	isr + 10
sig:	DB	"RotorDRV+FlipDRV"	; substitute signature string
        DB      0

onoff:  DB      0               ; our on/off flag

dispatch proc	near
	sti
        cmp     ah,7Fh          ; is it our on/off call?
        je      setonoff
        cmp     ah,7Eh          ; is it our get status call?
        je      getonoff
	cmp	ah,4		; is it a set target call?
	je	settarg
	cmp	ah,3		; is it a read angles call?
	je	readpos
	jmp	dword ptr old_isr	; no, just pass thru.

dispatch endp

;******************************************************************
; Function 7F: Set Flip Mode
; --------------------------------
; This function is used to set the RotorDRV to normal or inverted
; antennas mode.
;
;    At Entry: AH=7F
;              AL = 0 for normal antenna angles
;              AL = 1 for inverted antenna angles
;
;    At Exit:  No parameters are returned
;*******************************************************************

setonoff        proc    near

        mov     byte ptr cs:onoff,al    ; store the flag

        iret                            ; that's all.  No passthru.

setonoff        endp

;******************************************************************
; Function 7E: Get Flip Mode
; --------------------------------
; This function is used to get the RotorDRV mode, normal or inverted.
;
;    At Entry: AH=7E
;
;    At Exit:  AL = 0 for normal antenna angles
;              AL = 1 for inverted antenna angles
;*******************************************************************

getonoff        proc    near

        mov     al,byte ptr cs:onoff    ; get the flag

        iret                            ; that's all.  No passthru.

getonoff        endp


;******************************************************************
; Function 04: Set Target position
; --------------------------------
; This function is used by the TSR to tell the tracker TSR where to
; point the antennas.  It is called periodically.
;
;    At Entry: AH=4
;              CX - target ELEV value in degrees (-90 to 269, signed binary)
;              DX - target AZIM value in degrees (0-359, binary)
;
;    At Exit:  No parameters are returned
;*******************************************************************

settarg	proc	near

        cmp     byte ptr cs:onoff,0     ; are we flipping right now?
        je      st_exit

	neg	cx			; elevation = 180 - elevation
	add	cx,180

	add	dx,180			; azimuth = azimuth + 180 (mod 360)
	cmp	dx,360
	jb	lt360
	sub	dx,360
lt360:

				; pass thru to real RotorDRV
st_exit:	jmp	dword ptr old_isr
settarg endp


;**********************************************************************
;
;	This function is called to fill the registers with information
;	about the present and target antenna positions.  The register
;	definitions are as follows:
;
;		AX = current elevation in degrees 	(-90 to +269)
;		BX = current azimuth in degrees		(  0 to +359)
;		CX = target elevation in degrees	(-90 to +269)
;		DX = target azimuth in degrees		(  0 to +359)
;
;**********************************************************************

readpos	proc	near

	pushf			; call the real RotorDRV
	call	dword ptr old_isr	; fake up an INT call

        cmp     byte ptr cs:onoff,0     ; are we flipping right now?
        je      rp_exit

			; now translate back to normalized coordinates
				; target angles	
	neg	cx			; elevation = 180 - elevation
	add	cx,180

	add	dx,180			; azimuth = azimuth + 180 (mod 360)
	cmp	dx,360
	jb	lt360a
	sub	dx,360
lt360a:

				; current angles
	neg	ax			; elevation = 180 - elevation
	add	ax,180

	add	bx,180			; azimuth = azimuth + 180 (mod 360)
	cmp	bx,360
	jb	lt360b
	sub	bx,360
lt360b:


rp_exit:
	iret				; return modified values

readpos	endp

lastbyte	equ	this byte	; flag for end of memory used

vector		DB	63h		; default vector

banner_msg	DB	'FLIPDRV 0.6 Copyright 1993 Paul Williamson KB5MU'
		DB	13, 10, '$'

happy_msg	DB	'Installed.'
		DB	13, 10, '$'

duplicate_msg	DB	'FLIPDRV already installed!  Cannot reinstall.'
		DB	13, 10, '$'

notfound_msg	DB	'RotorDRV not installed at this vector.', 13, 10
		DB	'Cannot install FLIPDRV without a RotorDRV.'
		DB	13, 10, '$'

install	proc	near

	cld
	mov	ah,9
	mov	dx,OFFSET banner_msg
	int	21h

	call	get_vector		; find out what vector to install

	mov	ah, 35h			; get vector service
	mov	al, vector
	int	21h
	mov	WORD PTR old_isr,bx	; save it for passthru
	mov	WORD PTR old_isr+2,es

	mov	di,bx			; ES:DI points to old RotorDRV
	add	di,10			; ES:DI points to old signature
	mov	si,OFFSET sig		; DS:SI points to our signature
	mov	cx,8			; compare 8 characters
	repe	cmpsb			; do compare
	and	cx,cx			; test the result
	jnz	no_RotorDRV		; oops, didn't find a RotorDRV.
	mov	cx,8			; check for our own signature
	repe	cmpsb
	jnz	ok_to_install
	    				; FlipDRV is already present.
	mov	dx,OFFSET duplicate_msg
	jmp	err_out
	
no_RotorDRV:				; No RotorDRV was found.
	mov	dx,OFFSET notfound_msg

err_out:
	mov	ah,9			; print string function
	int	21h			; print the string
	mov	ax,4c01h		; return code = 1
	int	21h			; terminate (not installed)

ok_to_install:
	mov	dx, offset isr	; get address of service routine
	mov	ah, 25h		; set vector service
	mov	al, vector
	int	21h		; set up the interrupt vector

	mov	ah,9
	mov	dx,OFFSET happy_msg
	int	21h

				; reserve memory and terminate.
	mov	dx,offset lastbyte	; get last address used
	add	dx,15		; round up
	mov	cl,4
	shr	dx,cl		; convert to paragraphs
	mov	ax,3100h	; TSR function code, good status
	int	21H		; go TSR!

install	endp

;**********************************************************************
;
; This function gets the vector number.  It reads a decimal value from
; the command tail and loads that into vector.  If there is nothing
; in the command tail, it leaves vector unchanged.  If the command
; tail doesn't parse into exactly one decimal value, or if that value
; is out of range, it issues a message and refuses to install.
;
;**********************************************************************

gv_usage_msg	DB	'Usage: FlipDRV [vector number]', 13, 10
		DB	'If vector number is omitted, 99 (decimal) is used.', 13, 10
		DB	'Specify vector number in decimal.'
		DB	13, 10, '$'

gv_range_msg	DB	'Vector number must be between 47 and 256.'
		DB	13, 10, '$'

get_vector	PROC	NEAR

		mov	bx,80h		; address command tail count
		mov	cl,[bx]		; get command tail count
		cmp	cl,0		; is there a command tail?
		jz	gv_done		; no, use default vector.

		xor	ch,ch		; need CX for count
		xor	ax,ax		; start with zero
		mov	dh,10		; base multiplier
		mov	di,0		; digit count
gv_loop:
		inc	bx
		mov	dl,[bx]		; get next character
		cmp	dl,20h		; is it a space?
		je	gv_whitespace
		cmp	dl,9		; is it a tab?
		je	gv_whitespace
		cmp	dl,13		; is it a return?
		je	gv_endofline
		cmp	dl,0		; is it a null?
		je	gv_endofline
		cmp	dl,'0'		; is it too low for a digit?
		jl	gv_badchars
		cmp	dl,'9'		; is it a digit?
		jg	gv_badchars
		sub	dl,'0'		; convert to value
		mul	dh		; shift old value left 1 place
		add	al,dl		; add in new digit
		adc	ah,0		; overflow is bad, but let it happen.
		inc	di		; count digits
gv_nextchar:	loop	gv_loop		; go look for next digit.
gv_endofline:
		cmp	ax,47		; is value high enough?
		jle	gv_badvalue	; no.
		cmp	ax,100h		; is it too high?
		jge	gv_badvalue	; yes.
		mov	vector,al	; it's OK, so use it.

gv_done:	ret

gv_whitespace:		; if we find whitespace before a digit, just skip it.
			; if we find whitespace after a digit, the rest of
			; the line has to be whitespace.
		cmp	di,0		; have we seen a digit?
		je	gv_nextchar	; no, keep scanning.
					; yes, rest of line must be space.
gv_wsloop:
		inc	bx
		dec	cx
		mov	dl,[bx]		; get next character
		cmp	dl,20h		; is it a space?
		je	gv_wsnext	; yes, OK so far.
		cmp	dl,9		; is it a tab?
		je	gv_wsnext	; yes, OK so far.
		cmp	dl,13		; is it a return?
		je	gv_endofline	; yes, we've parsed the line now.
		cmp	dl,0		; is it a null?
		je	gv_endofline	; yes, we've parsed the line now.
		jmp	gv_badchars	; oops, more stuff on the line.

gv_wsnext:	loop	gv_wsloop
		jmp	gv_endofline	; fell off end, assume we're done.

gv_badchars:	mov	dx,OFFSET gv_usage_msg
gv_errmsg:	mov	ah,9		; display string
		int	21h

		mov	ax,4C03h	; return code = 3
		int	21h		; terminate (not installed).

gv_badvalue:	mov	dx,OFFSET gv_range_msg
		jmp	gv_errmsg

get_vector	ENDP

_TEXT	ends

	end	start
